#include <errno.h>
#include <unistd.h>
#include <ioctl-types.h>
#include <stdio.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/time.h>

#include "br_cmd.h"

#define VERSION "0.02"

/*

br (BottleRocket)

Control software for the X10(R) FireCracker wireless computer
interface kit.

(c) 1999 Tymm Twillman (tymm@acm.org).  Free Software.  LGPL applies.
  No warranties expressed or implied.

Have fun with it and if you do anything really cool, send an email and let me
know.


*/


/*
 * Yeah.  I know.  It'll get moved to ttySx soon enough... Maybe some day if 
 *   you're really lucky, I'll even add a command-line option.  Woo!
 */

#define X10_DEVICE "/dev/cua0"

char *MyName = "bottlerocket";   /* Default */

void usage()
{
	fprintf(stderr, "BottleRocket version %s                        \n", VERSION);
	fprintf(stderr, "(c) 1999 - Tymm Twillman                       \n");
	fprintf(stderr, "                                               \n");
	fprintf(stderr, "Purpose:  Control X10 \"FireCracker\" wireless \n");
	fprintf(stderr, "           home-control interface things.      \n");
	fprintf(stderr, "                                               \n");
	fprintf(stderr, "Usage: %s <A-P><1-16> (on | off)               \n",
	  MyName);
	fprintf(stderr, "       %s <A-P> (bright | dim)                 \n",
	  MyName);
	fprintf(stderr, "                                               \n");
	fprintf(stderr, "Examples: %s A1 on                             \n",
	  MyName);
	fprintf(stderr, "          %s P8 off                            \n",
	  MyName);
	fprintf(stderr, "          %s B dim                             \n",
	  MyName);
}

int get_cmd(char *arg)
{
	if (!strcasecmp("on", arg)) {
		return ON;
	} else if (!strcasecmp("off", arg)) {
		return OFF;
	} else if (!strcasecmp("dim", arg)) {
		return DIM;
	} else if (!strcasecmp("bright", arg)) {
		return BRIGHT;
	}

	return -1;
}

int get_device(char *arg, char cmd)
{
	char device;


	/*
	 * Figure out the numeric part of the address of a unit (which device
	 *  in a group) we want to talk to
	 *
	 * If the command is a "dim" command, the device should be set to 0.
	 *  Unfortunately, some of the address lines are used for the dim/
	 *  bright commands.
	 */

	if ((cmd == DIM) || (cmd == BRIGHT)) {
                if (strlen(arg) != 1)
			return -1;
		device = 0;
	} else {

		if (strlen(arg) < 2)
			return -1;
		device = (char)atoi(&arg[1]) - 1;
		if ((device < 0) || (device > 15))
			return -1;
	}

	return device;
}

int get_group(char *arg)
{
	/*
	 * Get the letter part of a unit address (what "group" of units 
	 *  requested one is in)
	 */

	int group;


	if (!strlen(arg))
		return -1;

	group = toupper(arg[0]) - 'A';
	if ((group > 15) || (group < 0))
		return -1;

	return group;
}
	
int main(int argc, char *argv[])
{
	char device;
	char group;
	int cmd;
 	int fd;
	unsigned short unit;


	/*
	 * Who am I?
	 */

	if (argc > 0)
		MyName = argv[0];

	/*
	 * What am I doing?
	 */
	
	if ((argc < 3)
	  || ((cmd = get_cmd(argv[2])) < 0)
	  || ((device = get_device(argv[1], cmd)) < 0)
	  || ((group = get_group(argv[1])) < 0)
	)
	{
		usage();
		exit(0);
	}

	unit = (unsigned char)((group << 4) | device);

	/*
	 * Snag the serial port...
	 */

	if ((fd = open(X10_DEVICE, O_RDWR)) < 0) {
		fprintf(stderr, "%s:  Error %d opening %s.\n", MyName, errno,
		  X10_DEVICE);
		perror(MyName);
		exit(errno);
	}

#ifdef DEBUG
	fprintf(stderr, "%s:  Opened %s.\n", MyName, X10_DEVICE);
	fflush(stdout);
#endif

	/*
	 * Do the deed...
	 */

	x10_br_out(fd, unit, (unsigned char)cmd);

	exit(0);
}
